#include "mapfile.h"
#include "zzlib.h"

LOGICO cargarMapa(char *nombre, unsigned char *flags, CADENA nombre_pcxs[],\
	CABECERA *cabecera, MAPA mapa, PALANCAS palancas, ENERGIAS energias,\
	BITMAP *fondos[], BITMAP *tiles[], BITMAP *sprites[],\
	int *n_efec)
{
	FILE *pf;
	int i,x,y;
	int dot;
	unsigned char byte;
	struct stat buf_stat;
	unsigned int longitudDescomprimido;
	int len;
	unsigned char *buffer;
	int offset = 0;
	
	*n_efec = -1;
	
	pf=fopen(nombre,"rb");
	
	if(pf==NULL)
		return(FALSO);
	
	
	fread(flags, sizeof(unsigned char), 1, pf);
	
	// Si est comprimido
	if(estaComprimido(*flags))
	{
		// Descomprimir y cargar
		stat(nombre,&buf_stat);
		len = buf_stat.st_size;
		buffer = (unsigned char *)malloc( len + 16 );
		
		// El entero de 32 bits que est al principio del buffer comprimido es
		// igual o mayor que el tamao del buffer descomprimido
		len = fread( buffer, sizeof(unsigned char), len, pf);
		longitudDescomprimido = *((unsigned int*)buffer);
		buffer = (unsigned char*)realloc( buffer, longitudDescomprimido );
		len = ZzUncompressBlock( buffer );
		
		// Ya hemos descomprimido. Ahora a copiar a lo bestia.
		memcpy(cabecera, buffer + offset, sizeof(CABECERA));
		offset += sizeof(CABECERA);
		memcpy(mapa, buffer + offset, sizeof(PANTALLA)*25);
		offset += sizeof(PANTALLA)*25;
		memcpy(palancas, buffer + offset, sizeof(OBJETO)*25);
		offset += sizeof(OBJETO)*25;
		memcpy(energias, buffer + offset, sizeof(OBJETO)*25);
		offset += sizeof(OBJETO)*25;
		memcpy(nombre_pcxs, buffer + offset, sizeof(CADENA)*7);
		offset += sizeof(CADENA)*7;
		
		for(i=0;i<5;i++)
		for(y=0;y<144;y++)
		for(x=0;x<240;x++)
		{
			memcpy(&dot, buffer + offset, sizeof(int));
			offset += sizeof(int);
			putpixel(fondos[i], x, y, dot);
		}
		
		for(i=0;i<240;i++)
		for(y=0;y<16;y++)
		for(x=0;x<16;x++)
		{
			memcpy(&dot, buffer + offset, sizeof(int));
			offset += sizeof(int);
			putpixel(tiles[i], x, y, dot);
			memcpy(&dot, buffer + offset, sizeof(int));
			offset += sizeof(int);
			putpixel(sprites[i], x, y, dot);
		}
		
		if(*flags & EFECTO == EFECTO)
		{
			memcpy(n_efec, buffer + offset, sizeof(int));
		}
		
		free(buffer);
	}
	else
	{
		// Slo cargar
		fread(cabecera,sizeof(CABECERA),1,pf);
		fread(mapa,sizeof(PANTALLA),25,pf);
		fread(palancas,sizeof(OBJETO),25,pf);
		fread(energias,sizeof(OBJETO),25,pf);
		
		fread(nombre_pcxs,sizeof(CADENA),7,pf);
		
		for(i=0;i<5;i++)
		for(y=0;y<144;y++)
		for(x=0;x<240;x++)
		{
			fread(&dot,sizeof(int),1,pf);
			putpixel(fondos[i],x,y,dot);
		}
		
		for(i=0;i<240;i++)
		for(y=0;y<16;y++)
		for(x=0;x<16;x++)
		{
			fread(&dot,sizeof(int),1,pf);
			putpixel(tiles[i],x,y,dot);
			fread(&dot,sizeof(int),1,pf);
			putpixel(sprites[i],x,y,dot);
		}
		
		if(tieneEfecto(*flags))
		{
			fread(n_efec, sizeof(int), 1, pf);
		}
	}
	
	fclose(pf);
	
	return(CIERTO);
}

LOGICO cargarMapaFormatoAntiguo(char *nombre,CADENA nombre_pcxs[],\
	CABECERA *cabecera, MAPA mapa, PALANCAS palancas, ENERGIAS energias,\
	BITMAP *fondos[], BITMAP *tiles[], BITMAP *sprites[],\
	int *n_efec)
{
	
	FILE *pf;
	int i,x,y;
	int dot;
	unsigned char byte;
	
	*n_efec = -1;
	
	pf=fopen(nombre,"rb");
	
	
	if(pf==NULL)
		return(FALSO);
	
	fread(cabecera,sizeof(CABECERA),1,pf);
	fread(mapa,sizeof(PANTALLA),25,pf);
	fread(palancas,sizeof(OBJETO),25,pf);
	fread(energias,sizeof(OBJETO),25,pf);
	
	fread(nombre_pcxs,sizeof(CADENA),7,pf);
	
	for(i=0;i<5;i++)
	for(y=0;y<144;y++)
	for(x=0;x<240;x++)
	{
		fread(&dot,sizeof(int),1,pf);
		putpixel(fondos[i],x,y,dot);
	}
	
	for(i=0;i<240;i++)
	for(y=0;y<16;y++)
	for(x=0;x<16;x++)
	{
		fread(&dot,sizeof(int),1,pf);
		putpixel(tiles[i],x,y,dot);
		fread(&dot,sizeof(int),1,pf);
		putpixel(sprites[i],x,y,dot);
	}
	
	fseek(pf,0,SEEK_END);
	
	if(ftell(pf)!=1211594)
	{
		fseek(pf,1211594,SEEK_SET);
		fread(n_efec,sizeof(int),1,pf);
	}
	
	fclose(pf);
	return(CIERTO);
}

LOGICO grabarMapa(char *nombre, unsigned char flags, CADENA nombre_pcxs[],\
	CABECERA cabecera, MAPA mapa, PALANCAS palancas, ENERGIAS energias,\
	BITMAP *fondos[], BITMAP *tiles[], BITMAP *sprites[],\
	int n_efec)
{
	FILE *pf;
	int i,x,y;
	int dot, extra = tieneEfecto(flags)?1:0;
	int cabZip = estaComprimido(flags)?16:0;
	unsigned char *buffer;
	int offset = 0;
	
	// Reservamos memoria para el buffer
	int len = sizeof(CABECERA) + \
	sizeof(PANTALLA)*25 + \
	sizeof(OBJETO)*50 + \
	sizeof(CADENA)*7 + \
	sizeof(int)*(295680+extra);
	
	if ( (buffer = (unsigned char *)malloc( len + cabZip )) == NULL )
	{
		return FALSO;
	}
	
	// Y ahora, en lugar de escribir en el fichero escribimos en el buffer
	
	memcpy(buffer + offset, &cabecera, sizeof(CABECERA));
	offset += sizeof(CABECERA);
	memcpy(buffer + offset, mapa, sizeof(PANTALLA)*25);
	offset += sizeof(PANTALLA)*25;
	memcpy(buffer + offset, palancas, sizeof(OBJETO)*25);
	offset += sizeof(OBJETO)*25;
	memcpy(buffer + offset, energias, sizeof(OBJETO)*25);
	offset += sizeof(OBJETO)*25;
	memcpy(buffer + offset, nombre_pcxs, sizeof(CADENA)*7);
	offset += sizeof(CADENA)*7;
	
	for(i=0;i<5;i++)
	for(y=0;y<144;y++)
	for(x=0;x<240;x++)
	{
		dot=getpixel(fondos[i],x,y);
		memcpy(buffer + offset, &dot, sizeof(int));
		offset += sizeof(int);
	}
	
	for(i=0;i<240;i++)
	for(y=0;y<16;y++)
	for(x=0;x<16;x++)
	{
		dot=getpixel(tiles[i],x,y);
		memcpy(buffer + offset, &dot, sizeof(int));
		offset += sizeof(int);
		dot=getpixel(sprites[i],x,y);
		memcpy(buffer + offset, &dot, sizeof(int));
		offset += sizeof(int);
	}
	
	if(n_efec!=-1)
	{
		memcpy(buffer + offset, &n_efec, sizeof(int));
	}
	pf=fopen(nombre,"wb");
	
	if(pf==NULL)
		return(FALSO);
	
	if(estaComprimido(flags))
	{
		len = ZzCompressBlock( buffer, len, 1, 0 );
	}
	
	if(len != -1 && len != -2)
	{
		fwrite( &flags, sizeof(unsigned char), 1, pf);
		fwrite( buffer, sizeof(unsigned char), len, pf);
	}
	
	free(buffer);
	fclose(pf);
	
	return(CIERTO);
}

LOGICO esEditable(unsigned char flags)
{
	return (flags & MAPA_EDITABLE) == MAPA_EDITABLE;
}

LOGICO esJugable(unsigned char flags)
{
	return (flags & JUGABLE) == JUGABLE;
}

LOGICO estaComprimido(unsigned char flags)
{
	return (flags & COMPRIMIDO) == COMPRIMIDO;
}


LOGICO tieneEfecto(unsigned char flags)
{
	return (flags & EFECTO) == EFECTO;
}

